@@ -279,7 +279,7 @@ recover_data.call = function(object, trms, na.action, data = NULL,
     if(!missing(pwts) && !is.null(pwts)) {
         if ((npw <- length(pwts)) == nrow(tbl))
             tbl[["(weights)"]] = pwts
-        else if (npw > 1)
+        else if (length(unique(pwts)) > 1)
             warning("Model has ", npw, " prior weights, but we recovered ",
                     nrow(tbl), " rows of data.\nSo prior weights were ignored.",
                     call. = FALSE)
@@ -385,7 +385,7 @@ recover_data.call = function(object, trms, na.action, data = NULL,
 #' \code{tol} is the tolerance for assessing estimability. It should return a
 #' matrix with 3 columns: the estimates, standard errors (\code{NA} when
 #' \code{do.se==FALSE}), and degrees of freedom (\code{NA} for asymptotic). The
-#' number of rows should be the same as \samp{object@linfct}. The
+#' number of rows should equal \samp{nrow(linfct(object)}. The
 #' \code{vcovHook} function should have arguments \samp{(object, tol, ...)} as
 #' described. It should return the covariance matrix for the estimates. Finally,
 #' \code{postGridHook}, if present, is called at the very end of
